@@ -28,8 +28,19 @@ GlmTest::GlmTest(const mv_Method *tm):tm(tm)
     Rlambda = gsl_matrix_alloc(tm->nVars, tm->nVars);
     Wj=gsl_matrix_alloc(tm->nRows, tm->nRows);
 
-    rnd=gsl_rng_alloc(gsl_rng_mt19937);    
-//    if (tm->reprand==TRUE) printf("GSL rng with repeated seeds\n");
+    // Initialize GSL rnd environment variables
+    const gsl_rng_type *T;
+    gsl_rng_env_setup();
+    T = gsl_rng_default;
+    // an mt19937 generator with a seed of 0
+    rnd = gsl_rng_alloc(T);
+    if (tm->reprand!=TRUE){
+       struct timeval tv;  // seed generation based on time
+       gettimeofday(&tv, 0);
+       unsigned long mySeed=tv.tv_sec + tv.tv_usec;
+       gsl_rng_set(rnd, mySeed);  // reset seed
+    }
+
     if ( tm->resamp==PERMUTE ) {
         permid = (unsigned int *)malloc(tm->nRows*sizeof(unsigned int));
 	for (unsigned int i=0; i<tm->nRows; i++) permid[i]=i;
@@ -160,6 +171,7 @@ int GlmTest::summary(glm *fit)
     clock_t clk_start=clock();
 
     for ( unsigned int i=0; i<tm->nboot; i++) {        
+        nSamp++;
         if ( tm->resamp==CASEBOOT ) 
              resampSmryCase(fit,bY,GrpXs,bO,i);
         else resampNonCase(fit, bY, i);
@@ -203,7 +215,6 @@ int GlmTest::summary(glm *fit)
            if ( *buj >= *suj ) *puj=*puj+1;
            calcAdjustP(tm->punit, nVars, buj+1, suj+1, puj+1, sortid[k]);
         } // end for j loop
-        nSamp++;
         // Prompts
         if ((tm->showtime==TRUE)&(i%100==0)) {
            diff=(float)(clock()-clk_start)/(float)CLOCKS_PER_SEC;
@@ -220,6 +231,7 @@ int GlmTest::summary(glm *fit)
            reinforceP( puj, nVars, sortid[k] );
     }  }
     // p = (#exceeding observed stat + 1)/(#nboot+1)
+//    printf("tm->nboot=%d, nSamp=%d\n", tm->nboot, nSamp);
     gsl_matrix_add_constant (Psmry, 1.0);
     gsl_matrix_scale (Psmry, (double)1.0/(nSamp+1));
 
@@ -359,6 +371,7 @@ int GlmTest::anova(glm *fit, gsl_matrix *isXvarIn)
            printf("Resampling begins for test %d.\n", i);
         for (j=0; j<tm->nboot; j++) {	
 //            printf("simu %d :", j);
+	    nSamp++;
 	    gsl_vector_set_zero (bStat);
 	    if ( tm->resamp == CASEBOOT ) {
                 resampAnovaCase(PtrAlt[mtype],bY,X1,bO,j);
@@ -394,7 +407,6 @@ int GlmTest::anova(glm *fit, gsl_matrix *isXvarIn)
            if ( *(buj) > (*(suj)-1e-8) ) *puj=*puj+1;
            // ------ get univariate counts ---------//            
            calcAdjustP(tm->punit,nVars,buj+1,suj+1,puj+1,sortid);
-	   nSamp++;
            // Prompts
            if ((tm->showtime==TRUE)&(j%100==0)) {
               dif = (float)(clock() - clk_start)/(float)CLOCKS_PER_SEC;
@@ -666,7 +678,6 @@ int GlmTest::resampSmryCase(glm *model, gsl_matrix *bT, GrpMat *GrpXs, gsl_matri
     if (bootID == NULL) {
        tXX = gsl_matrix_alloc(nParam, nParam);
        while (isValid==TRUE) { // if all isSingular==TRUE
-           if (tm->reprand!=TRUE) GetRNGstate();
            for (j=0; j<nRows; j++) {
                // resample Y, X, offsets accordingly
                if (tm->reprand==TRUE)
@@ -679,7 +690,6 @@ int GlmTest::resampSmryCase(glm *model, gsl_matrix *bT, GrpMat *GrpXs, gsl_matri
                oj = gsl_matrix_row(model->Eta, id);
                gsl_matrix_set_row(bO, j, &oj.vector);
            }
-           if (tm->reprand!=TRUE) PutRNGstate();
            gsl_matrix_set_identity(tXX);
            gsl_blas_dsyrk(CblasLower,CblasTrans,1.0,GrpXs[0].matrix,0.0,tXX);
            status=gsl_linalg_cholesky_decomp(tXX); 
@@ -723,7 +733,6 @@ int GlmTest::resampAnovaCase(glm *model, gsl_matrix *bT, gsl_matrix *bX, gsl_mat
 
     if (bootID == NULL) {
        while (isValid==TRUE) {
-            if (tm->reprand!=TRUE) GetRNGstate();
             for (j=0; j<nRows; j++) {   
                 if (tm->reprand==TRUE)
                    id=(unsigned int)gsl_rng_uniform_int(rnd, nRows);
@@ -736,7 +745,6 @@ int GlmTest::resampAnovaCase(glm *model, gsl_matrix *bT, gsl_matrix *bX, gsl_mat
                 gsl_matrix_set_row(bX, j, &xj.vector);
                 gsl_matrix_set_row(bO, j, &oj.vector);
              }
-             if (tm->reprand!=TRUE) PutRNGstate();
              gsl_matrix_set_identity(tXX);
              gsl_blas_dsyrk(CblasLower,CblasTrans,1.0,bX,0.0,tXX);
              status=gsl_linalg_cholesky_decomp(tXX); 
@@ -772,7 +780,6 @@ int GlmTest::resampNonCase(glm *model, gsl_matrix *bT, unsigned int i)
    // note that residuals have got means subtracted
    switch (tm->resamp) {
    case RESIBOOT: 
-       if (tm->reprand!=TRUE) GetRNGstate();
        for (j=0; j<nRows; j++) {
            if (bootID!=NULL)
                id = (unsigned int) gsl_matrix_get(bootID, i, j);
@@ -786,7 +793,6 @@ int GlmTest::resampNonCase(glm *model, gsl_matrix *bT, unsigned int i)
                bt = MIN(bt, model->maxtol);
                gsl_matrix_set(bT, j, k, bt);
         }   }
-        if (tm->reprand!=TRUE) PutRNGstate();   	  	
         break;
    case SCOREBOOT: 
         for (j=0; j<nRows; j++) {
@@ -831,7 +837,6 @@ int GlmTest::resampNonCase(glm *model, gsl_matrix *bT, unsigned int i)
         McSample(model, rnd, XBeta, Sigma, bT);
         break;
     case PITSBOOT:
-       if (tm->reprand!=TRUE) GetRNGstate();
        for (j=0; j<nRows; j++) {
            if (bootID!=NULL) 
                id = (unsigned int) gsl_matrix_get(bootID, i, j);
@@ -845,7 +850,6 @@ int GlmTest::resampNonCase(glm *model, gsl_matrix *bT, unsigned int i)
                gsl_matrix_set(bT, j, k, yij);
            }
        }
-       if (tm->reprand!=TRUE) PutRNGstate();
        break;
     default: GSL_ERROR("The resampling method is not supported", GSL_ERANGE); break;
     }
